@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ This is a thin wrapper around a pytree.Leaf node.
 import keyword
 import re
 
-from lib2to3 import pytree
 from lib2to3.pgen2 import token
 
 from yapf.yapflib import py3compat
@@ -47,11 +46,13 @@ class Subtype(object):
   KWARGS_STAR_STAR = 9
   ASSIGN_OPERATOR = 10
   DICTIONARY_KEY = 11
-  DICTIONARY_VALUE = 12
-  DICT_SET_GENERATOR = 13
-  COMP_FOR = 14
-  COMP_IF = 15
-  FUNC_DEF = 16
+  DICTIONARY_KEY_PART = 12
+  DICTIONARY_VALUE = 13
+  DICT_SET_GENERATOR = 14
+  COMP_FOR = 15
+  COMP_IF = 16
+  FUNC_DEF = 17
+  DECORATOR = 18
 
 
 class FormatToken(object):
@@ -103,6 +104,11 @@ class FormatToken(object):
     else:
       self.spaces_required_before = 0
 
+    if self.is_continuation:
+      self.value = self.node.value.rstrip()
+    else:
+      self.value = self.node.value
+
   def AddWhitespacePrefix(self, newlines_before, spaces=0, indent_level=0):
     """Register a token's whitespace prefix.
 
@@ -121,6 +127,9 @@ class FormatToken(object):
       comment_lines = [s.lstrip() for s in self.value.splitlines()]
       self.node.value = ('\n' + indent_before).join(comment_lines)
 
+      # Update our own value since we are changing node value
+      self.value = self.node.value
+
     if not self.whitespace_prefix:
       self.whitespace_prefix = (
           '\n' * (self.newlines or newlines_before) + indent_before)
@@ -175,12 +184,6 @@ class FormatToken(object):
     msg += ', pseudo)' if self.is_pseudo_paren else ')'
     return msg
 
-  @property
-  def value(self):
-    if self.is_continuation:
-      return self.node.value.rstrip()
-    return self.node.value
-
   @property
   @py3compat.lru_cache()
   def node_split_penalty(self):
@@ -275,4 +278,5 @@ class FormatToken(object):
 
   @property
   def is_pylint_comment(self):
-    return self.is_comment and re.match(r'#\s*\bpylint:\b', self.value)
+    return self.is_comment and re.match(r'#.*\bpylint:\s*(disable|enable)=',
+                                        self.value)
