@@ -74,8 +74,8 @@ def Reformat(uwlines, verify=False):
       _RetainHorizontalSpacing(uwline)
       _RetainVerticalSpacing(uwline, prev_uwline)
       _EmitLineUnformatted(state)
-    elif (_CanPlaceOnSingleLine(uwline) and not any(tok.must_split
-                                                    for tok in uwline.tokens)):
+    elif _CanPlaceOnSingleLine(uwline) and not any(tok.must_split
+                                                   for tok in uwline.tokens):
       # The unwrapped line fits on one line.
       while state.next_token:
         state.AddTokenToState(newline=False, dry_run=False)
@@ -230,6 +230,7 @@ def _CanPlaceOnSingleLine(uwline):
 
 
 def _FormatFinalLines(final_lines, verify):
+  """Compose the final output from the finalized lines."""
   formatted_code = []
   for line in final_lines:
     formatted_line = []
@@ -310,7 +311,6 @@ def _AnalyzeSolutionSpace(initial_state):
   heapq.heappush(p_queue, _QueueItem(_OrderedPenalty(0, count), node))
 
   count += 1
-  prev_penalty = 0
   while p_queue:
     item = p_queue[0]
     penalty = item.ordered_penalty.penalty
@@ -325,7 +325,6 @@ def _AnalyzeSolutionSpace(initial_state):
     if node.state in seen:
       continue
 
-    prev_penalty = penalty
     seen.add(node.state)
 
     # FIXME(morbo): Add a 'decision' element?
@@ -358,10 +357,10 @@ def _AddNextStateToQueue(penalty, previous_node, newline, count, p_queue):
   Returns:
     The updated number of elements in the queue.
   """
-  if newline and not previous_node.state.CanSplit():
+  must_split = previous_node.state.MustSplit()
+  if newline and not previous_node.state.CanSplit(must_split):
     # Don't add a newline if the token cannot be split.
     return count
-  must_split = previous_node.state.MustSplit()
   if not newline and must_split:
     # Don't add a token we must split but where we aren't splitting.
     return count
@@ -445,8 +444,11 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
     return 0
 
   if first_token.is_docstring:
+    if (prev_uwline.first.value == 'class' and
+        style.Get('BLANK_LINE_BEFORE_CLASS_DOCSTRING')):
+      # Enforce a blank line before a class's docstring.
+      return ONE_BLANK_LINE
     # The docstring shouldn't have a newline before it.
-    # TODO(morbo): Add a knob to adjust this.
     return NO_BLANK_LINES
 
   prev_last_token = prev_uwline.last
